class CreatePageRevisions < ActiveRecord::Migration
  def up
    create_table :page_revisions do |t|
      t.string   "title_no"
      t.string   "title_en"
      t.text     "content_no"
      t.text     "content_en"
      t.string   "content_type", null: false, default: "markdown"

      t.integer  "page_id", null: false
      t.integer  "version", null: false, default: 1

      t.integer  "member_id"

      t.timestamps
    end

    add_foreign_key :page_revisions, :pages 
    add_foreign_key :page_revisions, :members, primary_key: :medlem_id

    add_index :page_revisions, [:page_id, :version], unique: true

    execute <<-SQL
      INSERT INTO page_revisions (
        page_id, title_no, title_en, content_no, content_en, content_type, created_at, updated_at
      )
      SELECT
        id, title_no, title_en, content_no, content_en, content_type, NOW(), NOW()
      FROM pages
    SQL

    remove_column :pages, :title_no
    remove_column :pages, :title_en
    remove_column :pages, :content_no
    remove_column :pages, :content_en
    remove_column :pages, :content_type
  end

  def down
    add_column :pages, :title_no, :string
    add_column :pages, :title_en, :string
    add_column :pages, :content_no, :text
    add_column :pages, :content_en, :text
    add_column :pages, :content_type, :string, null: false, default: "markdown"

    execute <<-SQL
      UPDATE pages
      SET
        title_no     = latest.title_no,
        title_en     = latest.title_en,
        content_no   = latest.content_no,
        content_en   = latest.content_en,
        content_type = latest.content_type,
        updated_at   = latest.updated_at
      FROM (
        SELECT
          page_id, title_no, title_en, content_no, content_en, content_type, updated_at
        FROM page_revisions
        NATURAL INNER JOIN (
          SELECT page_id, MAX(version) AS version
          FROM page_revisions
          GROUP BY page_id
        ) AS latest_version_per_page
      ) AS latest
      WHERE id = latest.page_id
    SQL

    drop_table :page_revisions
  end
end
